@@ -53,7 +53,6 @@ struct connman_device {
 							 */
 	bool powered;
 	bool scanning;
-	bool disconnected;
 	char *name;
 	char *node;
 	char *address;
@@ -588,7 +587,6 @@ int connman_device_set_powered(struct connman_device *device,
 
 	__connman_technology_enabled(type);
 
-	connman_device_set_disconnected(device, false);
 	device->scanning = false;
 
 	if (device->driver && device->driver->scan)
@@ -623,8 +621,6 @@ int __connman_device_disconnect(struct connman_device *device)
 
 	DBG("device %p", device);
 
-	connman_device_set_disconnected(device, true);
-
 	g_hash_table_iter_init(&iter, device->networks);
 
 	while (g_hash_table_iter_next(&iter, &key, &value)) {
@@ -675,7 +671,8 @@ static void mark_network_unavailable(gpointer key, gpointer value,
 	struct connman_network *network = value;
 
 	if (connman_network_get_connected(network) ||
-			connman_network_get_connecting(network))
+			connman_network_get_connecting(network) ||
+			connman_network_get_connectable(network))
 		return;
 
 	connman_network_set_available(network, false);
@@ -692,6 +689,9 @@ static gboolean remove_unavailable_network(gpointer key, gpointer value,
 	if (connman_network_get_available(network))
 		return FALSE;
 
+	if (connman_network_get_connectable(network))
+		return FALSE;
+
 	return TRUE;
 }
 
@@ -750,37 +750,6 @@ int connman_device_set_scanning(struct connman_device *device,
 	return 0;
 }
 
-/**
- * connman_device_set_disconnected:
- * @device: device structure
- * @disconnected: disconnected state
- *
- * Change disconnected state of device (only for device with networks)
- */
-int connman_device_set_disconnected(struct connman_device *device,
-						bool disconnected)
-{
-	DBG("device %p disconnected %d", device, disconnected);
-
-	if (device->disconnected == disconnected)
-		return -EALREADY;
-
-	device->disconnected = disconnected;
-
-	return 0;
-}
-
-/**
- * connman_device_get_disconnected:
- * @device: device structure
- *
- * Get device disconnected state
- */
-bool connman_device_get_disconnected(struct connman_device *device)
-{
-	return device->disconnected;
-}
-
 /**
  * connman_device_set_string:
  * @device: device structure
@@ -1352,6 +1321,11 @@ nodevice:
 	}
 
 list:
+	if (__connman_inet_isrootnfs_device(devname)) {
+		DBG("ignoring device %s (rootnfs)", devname);
+		return true;
+	}
+
 	blacklisted_interfaces =
 		connman_setting_get_string_list("NetworkInterfaceBlacklist");
 	if (!blacklisted_interfaces)
