@@ -41,6 +41,8 @@
 
 #include "connman.h"
 
+#define debug(fmt...) do { } while (0)
+
 #if __BYTE_ORDER == __LITTLE_ENDIAN
 struct domain_hdr {
 	uint16_t id;
@@ -282,7 +284,7 @@ static struct server_data *find_server(int index,
 {
 	GSList *list;
 
-	DBG("index %d server %s proto %d", index, server, protocol);
+	debug("index %d server %s proto %d", index, server, protocol);
 
 	for (list = server_list; list; list = list->next) {
 		struct server_data *data = list->data;
@@ -333,14 +335,14 @@ static void refresh_dns_entry(struct cache_entry *entry, char *name)
 	}
 
 	if (!entry->ipv4) {
-		DBG("Refresing A record for %s", name);
+		debug("Refreshing A record for %s", name);
 		g_resolv_lookup_hostname(ipv4_resolve, name,
 					dummy_resolve_func, NULL);
 		age = 4;
 	}
 
 	if (!entry->ipv6) {
-		DBG("Refresing AAAA record for %s", name);
+		debug("Refreshing AAAA record for %s", name);
 		g_resolv_lookup_hostname(ipv6_resolve, name,
 					dummy_resolve_func, NULL);
 		age = 4;
@@ -355,7 +357,7 @@ static int dns_name_length(unsigned char *buf)
 {
 	if ((buf[0] & NS_CMPRSFLGS) == NS_CMPRSFLGS) /* compressed name */
 		return 2;
-	return strlen((char *)buf);
+	return strlen((char *)buf) + 1;
 }
 
 static void update_cached_ttl(unsigned char *buf, int len, int new_ttl)
@@ -452,7 +454,7 @@ static void send_cached_response(int sk, unsigned char *buf, int len,
 	else
 		update_cached_ttl((unsigned char *)hdr, adj_len, ttl);
 
-	DBG("sk %d id 0x%04x answers %d ptr %p length %d dns %d",
+	debug("sk %d id 0x%04x answers %d ptr %p length %d dns %d",
 		sk, hdr->id, answers, ptr, len, dns_len);
 
 	err = sendto(sk, ptr, len, MSG_NOSIGNAL, to, tolen);
@@ -464,7 +466,7 @@ static void send_cached_response(int sk, unsigned char *buf, int len,
 
 	if (err != len || (dns_len != (len - 2) && protocol == IPPROTO_TCP) ||
 				(dns_len != len && protocol == IPPROTO_UDP))
-		DBG("Packet length mismatch, sent %d wanted %d dns %d",
+		debug("Packet length mismatch, sent %d wanted %d dns %d",
 			err, len, dns_len);
 }
 
@@ -475,7 +477,7 @@ static void send_response(int sk, unsigned char *buf, int len,
 	struct domain_hdr *hdr;
 	int err, offset = protocol_offset(protocol);
 
-	DBG("sk %d", sk);
+	debug("sk %d", sk);
 
 	if (offset < 0)
 		return;
@@ -485,7 +487,7 @@ static void send_response(int sk, unsigned char *buf, int len,
 
 	hdr = (void *) (buf + offset);
 
-	DBG("id 0x%04x qr %d opcode %d", hdr->id, hdr->qr, hdr->opcode);
+	debug("id 0x%04x qr %d opcode %d", hdr->id, hdr->qr, hdr->opcode);
 
 	hdr->qr = 1;
 	hdr->rcode = ns_r_servfail;
@@ -537,7 +539,7 @@ static gboolean request_timeout(gpointer user_data)
 	if (!req)
 		return FALSE;
 
-	DBG("id 0x%04x", req->srcid);
+	debug("id 0x%04x", req->srcid);
 
 	request_list = g_slist_remove(request_list, req);
 
@@ -579,7 +581,7 @@ static gboolean request_timeout(gpointer user_data)
 	 * if we get a request timeout from server.
 	 */
 	if (req->protocol == IPPROTO_TCP) {
-		DBG("client %d removed", req->client_sk);
+		debug("client %d removed", req->client_sk);
 		g_hash_table_remove(partial_tcp_req_table,
 				GINT_TO_POINTER(req->client_sk));
 	}
@@ -597,7 +599,7 @@ static int append_query(unsigned char *buf, unsigned int size,
 	unsigned char *ptr = buf;
 	int len;
 
-	DBG("query %s domain %s", query, domain);
+	debug("query %s domain %s", query, domain);
 
 	while (query) {
 		const char *tmp;
@@ -667,7 +669,7 @@ static void cache_enforce_validity(struct cache_entry *entry)
 
 	if (!cache_check_is_valid(entry->ipv4, current_time)
 							&& entry->ipv4) {
-		DBG("cache timeout \"%s\" type A", entry->key);
+		debug("cache timeout \"%s\" type A", entry->key);
 		g_free(entry->ipv4->data);
 		g_free(entry->ipv4);
 		entry->ipv4 = NULL;
@@ -676,7 +678,7 @@ static void cache_enforce_validity(struct cache_entry *entry)
 
 	if (!cache_check_is_valid(entry->ipv6, current_time)
 							&& entry->ipv6) {
-		DBG("cache timeout \"%s\" type AAAA", entry->key);
+		debug("cache timeout \"%s\" type AAAA", entry->key);
 		g_free(entry->ipv6->data);
 		g_free(entry->ipv6);
 		entry->ipv6 = NULL;
@@ -701,7 +703,7 @@ static uint16_t cache_check_validity(char *question, uint16_t type,
 	switch (type) {
 	case 1:		/* IPv4 */
 		if (!cache_check_is_valid(entry->ipv4, current_time)) {
-			DBG("cache %s \"%s\" type A", entry->ipv4 ?
+			debug("cache %s \"%s\" type A", entry->ipv4 ?
 					"timeout" : "entry missing", question);
 
 			if (want_refresh)
@@ -720,7 +722,7 @@ static uint16_t cache_check_validity(char *question, uint16_t type,
 
 	case 28:	/* IPv6 */
 		if (!cache_check_is_valid(entry->ipv6, current_time)) {
-			DBG("cache %s \"%s\" type AAAA", entry->ipv6 ?
+			debug("cache %s \"%s\" type AAAA", entry->ipv6 ?
 					"timeout" : "entry missing", question);
 
 			if (want_refresh)
@@ -766,7 +768,7 @@ static gboolean try_remove_cache(gpointer user_data)
 	cache_timer = 0;
 
 	if (__sync_fetch_and_sub(&cache_refcount, 1) == 1) {
-		DBG("No cache users, removing it.");
+		debug("No cache users, removing it.");
 
 		g_hash_table_destroy(cache);
 		cache = NULL;
@@ -980,7 +982,7 @@ static int parse_response(unsigned char *buf, int buflen,
 	if (buflen < 12)
 		return -EINVAL;
 
-	DBG("qr %d qdcount %d", hdr->qr, qdcount);
+	debug("qr %d qdcount %d", hdr->qr, qdcount);
 
 	/* We currently only cache responses where question count is 1 */
 	if (hdr->qr != 1 || qdcount != 1)
@@ -1225,7 +1227,7 @@ static void cache_cleanup(void)
 		count = g_hash_table_foreach_remove(cache, cache_check_entry,
 						&data);
 	}
-	DBG("removed %d in the first pass", count);
+	debug("removed %d in the first pass", count);
 
 	/*
 	 * In the second pass, if the first pass turned up blank,
@@ -1289,7 +1291,7 @@ static gboolean cache_invalidate_entry(gpointer key, gpointer value,
  */
 static void cache_invalidate(void)
 {
-	DBG("Invalidating the DNS cache %p", cache);
+	debug("Invalidating the DNS cache %p", cache);
 
 	if (!cache)
 		return;
@@ -1321,7 +1323,7 @@ static void cache_refresh_entry(struct cache_entry *entry)
 			*c = '.';
 			c += jump + 1;
 		}
-		DBG("Refreshing %s\n", dns_name);
+		debug("Refreshing %s\n", dns_name);
 		/* then refresh the hostname */
 		refresh_dns_entry(entry, &dns_name[1]);
 	}
@@ -1357,7 +1359,7 @@ static int reply_query_type(unsigned char *msg, int len)
 		return 0;
 
 	/* now the query, which is a name and 2 16 bit words */
-	l = dns_name_length(c) + 1;
+	l = dns_name_length(c);
 	c += l;
 	type = c[0] << 8 | c[1];
 
@@ -1398,7 +1400,7 @@ static int cache_update(struct server_data *srv, unsigned char *msg,
 	if (offset < 0)
 		return 0;
 
-	DBG("offset %d hdr %p msg %p rcode %d", offset, hdr, msg, hdr->rcode);
+	debug("offset %d hdr %p msg %p rcode %d", offset, hdr, msg, hdr->rcode);
 
 	/* Continue only if response code is 0 (=ok) */
 	if (hdr->rcode != ns_r_noerror)
@@ -1578,7 +1580,7 @@ static int cache_update(struct server_data *srv, unsigned char *msg,
 		cache_size++;
 	}
 
-	DBG("cache %d %squestion \"%s\" type %d ttl %d size %zd packet %u "
+	debug("cache %d %squestion \"%s\" type %d ttl %d size %zd packet %u "
 								"dns len %u",
 		cache_size, new_entry ? "new " : "old ",
 		question, type, ttl,
@@ -1604,7 +1606,7 @@ static int ns_resolv(struct server_data *server, struct request_data *req,
 		int ttl_left = 0;
 		struct cache_data *data;
 
-		DBG("cache hit %s type %s", lookup, type == 1 ? "A" : "AAAA");
+		debug("cache hit %s type %s", lookup, type == 1 ? "A" : "AAAA");
 		if (type == 1)
 			data = entry->ipv4;
 		else
@@ -1641,7 +1643,7 @@ static int ns_resolv(struct server_data *server, struct request_data *req,
 	err = sendto(sk, request, req->request_len, MSG_NOSIGNAL,
 			server->server_addr, server->server_addr_len);
 	if (err < 0) {
-		DBG("Cannot send message to server %s sock %d "
+		debug("Cannot send message to server %s sock %d "
 			"protocol %d (%s/%d)",
 			server->server, sk, server->protocol,
 			strerror(errno), errno);
@@ -1701,7 +1703,7 @@ static int ns_resolv(struct server_data *server, struct request_data *req,
 			alt[1] = req_len & 0xff;
 		}
 
-		DBG("req %p dstid 0x%04x altid 0x%04x", req, req->dstid,
+		debug("req %p dstid 0x%04x altid 0x%04x", req, req->dstid,
 				req->altid);
 
 		err = send(sk, alt, req->request_len + domlen, MSG_NOSIGNAL);
@@ -1723,7 +1725,7 @@ static char *convert_label(char *start, char *end, char *ptr, char *uptr,
 	pos = dn_expand((u_char *)start, (u_char *)end, (u_char *)ptr,
 			name, NS_MAXLABEL);
 	if (pos < 0) {
-		DBG("uncompress error [%d/%s]", errno, strerror(errno));
+		debug("uncompress error [%d/%s]", errno, strerror(errno));
 		goto out;
 	}
 
@@ -1733,7 +1735,7 @@ static char *convert_label(char *start, char *end, char *ptr, char *uptr,
 	 */
 	comp_pos = dn_comp(name, (u_char *)uptr, remaining_len, NULL, NULL);
 	if (comp_pos < 0) {
-		DBG("compress error [%d/%s]", errno, strerror(errno));
+		debug("compress error [%d/%s]", errno, strerror(errno));
 		goto out;
 	}
 
@@ -1752,7 +1754,7 @@ static char *uncompress(int16_t field_count, char *start, char *end,
 {
 	char *uptr = *uncompressed_ptr; /* position in result buffer */
 
-	DBG("count %d ptr %p end %p uptr %p", field_count, ptr, end, uptr);
+	debug("count %d ptr %p end %p uptr %p", field_count, ptr, end, uptr);
 
 	while (field_count-- > 0 && ptr < end) {
 		int dlen;		/* data field length */
@@ -1774,7 +1776,7 @@ static char *uncompress(int16_t field_count, char *start, char *end,
 		ulen = strlen(name);
 		strncpy(uptr, name, uncomp_len - (uptr - uncompressed));
 
-		DBG("pos %d ulen %d left %d name %s", pos, ulen,
+		debug("pos %d ulen %d left %d name %s", pos, ulen,
 			(int)(uncomp_len - (uptr - uncompressed)), uptr);
 
 		uptr += ulen;
@@ -1818,7 +1820,7 @@ static char *uncompress(int16_t field_count, char *start, char *end,
 			dlen = uptr[-2] << 8 | uptr[-1];
 
 			if (ptr + dlen > end) {
-				DBG("data len %d too long", dlen);
+				debug("data len %d too long", dlen);
 				goto out;
 			}
 
@@ -1929,13 +1931,13 @@ static int forward_dns_reply(unsigned char *reply, int reply_len, int protocol,
 	hdr = (void *)(reply + offset);
 	dns_id = reply[offset] | reply[offset + 1] << 8;
 
-	DBG("Received %d bytes (id 0x%04x)", reply_len, dns_id);
+	debug("Received %d bytes (id 0x%04x)", reply_len, dns_id);
 
 	req = find_request(dns_id);
 	if (!req)
 		return -EINVAL;
 
-	DBG("req %p dstid 0x%04x altid 0x%04x rcode %d",
+	debug("req %p dstid 0x%04x altid 0x%04x rcode %d",
 			req, req->dstid, req->altid, hdr->rcode);
 
 	reply[offset] = req->srcid & 0xff;
@@ -1991,7 +1993,7 @@ static int forward_dns_reply(unsigned char *reply, int reply_len, int protocol,
 							ptr[dns_type_pos + 3];
 			if (dns_type != ns_t_a && dns_type != ns_t_aaaa &&
 					dns_class != ns_c_in) {
-				DBG("Pass msg dns type %d class %d",
+				debug("Pass msg dns type %d class %d",
 					dns_type, dns_class);
 				goto pass;
 			}
@@ -2050,21 +2052,21 @@ static int forward_dns_reply(unsigned char *reply, int reply_len, int protocol,
 						(char *)reply + offset, eom,
 						ptr, uncompressed, NS_MAXDNAME,
 						&uptr);
-				if (ptr == NULL)
+				if (!ptr)
 					goto out;
 
 				ptr = uncompress(ntohs(hdr->nscount),
 						(char *)reply + offset, eom,
 						ptr, uncompressed, NS_MAXDNAME,
 						&uptr);
-				if (ptr == NULL)
+				if (!ptr)
 					goto out;
 
 				ptr = uncompress(ntohs(hdr->arcount),
 						(char *)reply + offset, eom,
 						ptr, uncompressed, NS_MAXDNAME,
 						&uptr);
-				if (ptr == NULL)
+				if (!ptr)
 					goto out;
 
 				/*
@@ -2080,7 +2082,7 @@ static int forward_dns_reply(unsigned char *reply, int reply_len, int protocol,
 				new_len = strip_domains(uncompressed, answers,
 							uptr - answers);
 				if (new_len < 0) {
-					DBG("Corrupted packet");
+					debug("Corrupted packet");
 					return -EINVAL;
 				}
 
@@ -2145,10 +2147,10 @@ out:
 	}
 
 	if (err < 0)
-		DBG("Cannot send msg, sk %d proto %d errno %d/%s", sk,
+		debug("Cannot send msg, sk %d proto %d errno %d/%s", sk,
 			protocol, errno, strerror(errno));
 	else
-		DBG("proto %d sent %d bytes to %d", protocol, err, sk);
+		debug("proto %d sent %d bytes to %d", protocol, err, sk);
 
 	destroy_request_data(req);
 
@@ -2157,7 +2159,7 @@ out:
 
 static void server_destroy_socket(struct server_data *data)
 {
-	DBG("index %d server %s proto %d", data->index,
+	debug("index %d server %s proto %d", data->index,
 					data->server, data->protocol);
 
 	if (data->watch > 0) {
@@ -2182,7 +2184,7 @@ static void server_destroy_socket(struct server_data *data)
 
 static void destroy_server(struct server_data *server)
 {
-	DBG("index %d server %s sock %d", server->index, server->server,
+	debug("index %d server %s sock %d", server->index, server->server,
 			server->channel ?
 			g_io_channel_unix_get_fd(server->channel): -1);
 
@@ -2190,7 +2192,7 @@ static void destroy_server(struct server_data *server)
 	server_destroy_socket(server);
 
 	if (server->protocol == IPPROTO_UDP && server->enabled)
-		DBG("Removing DNS server %s", server->server);
+		debug("Removing DNS server %s", server->server);
 
 	g_free(server->server);
 	g_list_free_full(server->domains, g_free);
@@ -2250,7 +2252,7 @@ static gboolean tcp_server_event(GIOChannel *channel, GIOCondition condition,
 	if (condition & (G_IO_NVAL | G_IO_ERR | G_IO_HUP)) {
 		GSList *list;
 hangup:
-		DBG("TCP server channel closed, sk %d", sk);
+		debug("TCP server channel closed, sk %d", sk);
 
 		/*
 		 * Discard any partial response which is buffered; better
@@ -2259,9 +2261,11 @@ hangup:
 		g_free(server->incoming_reply);
 		server->incoming_reply = NULL;
 
-		for (list = request_list; list; list = list->next) {
+		list = request_list;
+		while (list) {
 			struct request_data *req = list->data;
 			struct domain_hdr *hdr;
+			list = list->next;
 
 			if (req->protocol == IPPROTO_UDP)
 				continue;
@@ -2303,7 +2307,7 @@ hangup:
 						domains = domains->next) {
 				char *dom = domains->data;
 
-				DBG("Adding domain %s to %s",
+				debug("Adding domain %s to %s",
 						dom, server->server);
 
 				server->domains = g_list_append(server->domains,
@@ -2328,7 +2332,7 @@ hangup:
 				continue;
 			}
 
-			DBG("Sending req %s over TCP", (char *)req->name);
+			debug("Sending req %s over TCP", (char *)req->name);
 
 			status = ns_resolv(server, req,
 						req->request, req->name);
@@ -2387,7 +2391,7 @@ hangup:
 			reply_len = reply_len_buf[1] | reply_len_buf[0] << 8;
 			reply_len += 2;
 
-			DBG("TCP reply %d bytes from %d", reply_len, sk);
+			debug("TCP reply %d bytes from %d", reply_len, sk);
 
 			reply = g_try_malloc(sizeof(*reply) + reply_len + 2);
 			if (!reply)
@@ -2434,7 +2438,7 @@ static gboolean tcp_idle_timeout(gpointer user_data)
 {
 	struct server_data *server = user_data;
 
-	DBG("");
+	debug("");
 
 	if (!server)
 		return FALSE;
@@ -2449,7 +2453,7 @@ static int server_create_socket(struct server_data *data)
 	int sk, err;
 	char *interface;
 
-	DBG("index %d server %s proto %d", data->index,
+	debug("index %d server %s proto %d", data->index,
 					data->server, data->protocol);
 
 	sk = socket(data->server_addr->sa_family,
@@ -2463,7 +2467,7 @@ static int server_create_socket(struct server_data *data)
 		return -err;
 	}
 
-	DBG("sk %d", sk);
+	debug("sk %d", sk);
 
 	interface = connman_inet_ifname(data->index);
 	if (interface) {
@@ -2523,6 +2527,25 @@ static int server_create_socket(struct server_data *data)
 	return 0;
 }
 
+static void enable_fallback(bool enable)
+{
+	GSList *list;
+
+	for (list = server_list; list; list = list->next) {
+		struct server_data *data = list->data;
+
+		if (data->index != -1)
+			continue;
+
+		if (enable)
+			DBG("Enabling fallback DNS server %s", data->server);
+		else
+			DBG("Disabling fallback DNS server %s", data->server);
+
+		data->enabled = enable;
+	}
+}
+
 static struct server_data *create_server(int index,
 					const char *domain, const char *server,
 					int protocol)
@@ -2609,6 +2632,8 @@ static struct server_data *create_server(int index,
 								data->index)) {
 			data->enabled = true;
 			DBG("Adding DNS server %s", data->server);
+
+			enable_fallback(false);
 		}
 
 		server_list = g_slist_append(server_list, data);
@@ -2630,7 +2655,7 @@ static bool resolv(struct request_data *req,
 			continue;
 		}
 
-		DBG("server %s enabled %d", data->server, data->enabled);
+		debug("server %s enabled %d", data->server, data->enabled);
 
 		if (!data->enabled)
 			continue;
@@ -2649,7 +2674,7 @@ static bool resolv(struct request_data *req,
 	return false;
 }
 
-static void append_domain(int index, const char *domain)
+static void update_domain(int index, const char *domain, bool append)
 {
 	GSList *list;
 
@@ -2680,13 +2705,55 @@ static void append_domain(int index, const char *domain)
 			}
 		}
 
-		if (!dom_found) {
+		if (!dom_found && append) {
 			data->domains =
 				g_list_append(data->domains, g_strdup(domain));
+		} else if (dom_found && !append) {
+			data->domains =
+				g_list_remove(data->domains, dom);
+			g_free(dom);
 		}
 	}
 }
 
+static void append_domain(int index, const char *domain)
+{
+	update_domain(index, domain, true);
+}
+
+static void remove_domain(int index, const char *domain)
+{
+	update_domain(index, domain, false);
+}
+
+static void flush_requests(struct server_data *server)
+{
+	GSList *list;
+
+	list = request_list;
+	while (list) {
+		struct request_data *req = list->data;
+
+		list = list->next;
+
+		if (ns_resolv(server, req, req->request, req->name)) {
+			/*
+			 * A cached result was sent,
+			 * so the request can be released
+			 */
+			request_list =
+				g_slist_remove(request_list, req);
+			destroy_request_data(req);
+			continue;
+		}
+
+		if (req->timeout > 0)
+			g_source_remove(req->timeout);
+
+		req->timeout = g_timeout_add_seconds(5, request_timeout, req);
+	}
+}
+
 int __connman_dnsproxy_append(int index, const char *domain,
 							const char *server)
 {
@@ -2719,6 +2786,8 @@ int __connman_dnsproxy_append(int index, const char *domain,
 	if (!data)
 		return -EIO;
 
+	flush_requests(data);
+
 	return 0;
 }
 
@@ -2726,12 +2795,22 @@ static void remove_server(int index, const char *domain,
 			const char *server, int protocol)
 {
 	struct server_data *data;
+	GSList *list;
 
 	data = find_server(index, server, protocol);
 	if (!data)
 		return;
 
 	destroy_server(data);
+
+	for (list = server_list; list; list = list->next) {
+		struct server_data *data = list->data;
+
+		if (data->index != -1 && data->enabled == true)
+			return;
+	}
+
+	enable_fallback(true);
 }
 
 int __connman_dnsproxy_remove(int index, const char *domain,
@@ -2739,9 +2818,15 @@ int __connman_dnsproxy_remove(int index, const char *domain,
 {
 	DBG("index %d server %s", index, server);
 
-	if (!server)
+	if (!server && !domain)
 		return -EINVAL;
 
+	if (!server) {
+		remove_domain(index, domain);
+
+		return 0;
+	}
+
 	if (g_str_equal(server, "127.0.0.1"))
 		return -ENODEV;
 
@@ -2754,33 +2839,6 @@ int __connman_dnsproxy_remove(int index, const char *domain,
 	return 0;
 }
 
-void __connman_dnsproxy_flush(void)
-{
-	GSList *list;
-
-	list = request_list;
-	while (list) {
-		struct request_data *req = list->data;
-
-		list = list->next;
-
-		if (resolv(req, req->request, req->name)) {
-			/*
-			 * A cached result was sent,
-			 * so the request can be released
-			 */
-			request_list =
-				g_slist_remove(request_list, req);
-			destroy_request_data(req);
-			continue;
-		}
-
-		if (req->timeout > 0)
-			g_source_remove(req->timeout);
-		req->timeout = g_timeout_add_seconds(5, request_timeout, req);
-	}
-}
-
 static void dnsproxy_offline_mode(bool enabled)
 {
 	GSList *list;
@@ -2805,6 +2863,7 @@ static void dnsproxy_offline_mode(bool enabled)
 
 static void dnsproxy_default_changed(struct connman_service *service)
 {
+	bool server_enabled = false;
 	GSList *list;
 	int index;
 
@@ -2829,12 +2888,16 @@ static void dnsproxy_default_changed(struct connman_service *service)
 		if (data->index == index) {
 			DBG("Enabling DNS server %s", data->server);
 			data->enabled = true;
+			server_enabled = true;
 		} else {
 			DBG("Disabling DNS server %s", data->server);
 			data->enabled = false;
 		}
 	}
 
+	if (!server_enabled)
+		enable_fallback(true);
+
 	cache_refresh();
 }
 
@@ -2859,7 +2922,7 @@ static int parse_request(unsigned char *buf, int len,
 	if (len < 12)
 		return -EINVAL;
 
-	DBG("id 0x%04x qr %d opcode %d qdcount %d arcount %d",
+	debug("id 0x%04x qr %d opcode %d qdcount %d arcount %d",
 					hdr->id, hdr->qr, hdr->opcode,
 							qdcount, arcount);
 
@@ -2897,7 +2960,7 @@ static int parse_request(unsigned char *buf, int len,
 
 		edns0_bufsize = last_label[7] << 8 | last_label[8];
 
-		DBG("EDNS0 buffer size %u", edns0_bufsize);
+		debug("EDNS0 buffer size %u", edns0_bufsize);
 
 		/* This is an evil hack until full TCP support has been
 		 * implemented.
@@ -2913,7 +2976,7 @@ static int parse_request(unsigned char *buf, int len,
 		}
 	}
 
-	DBG("query %s", name);
+	debug("query %s", name);
 
 	return 0;
 }
@@ -2924,7 +2987,7 @@ static void client_reset(struct tcp_partial_client_data *client)
 		return;
 
 	if (client->channel) {
-		DBG("client %d closing",
+		debug("client %d closing",
 			g_io_channel_unix_get_fd(client->channel));
 
 		g_io_channel_unref(client->channel);
@@ -2968,14 +3031,14 @@ static bool read_tcp_data(struct tcp_partial_client_data *client,
 	client_sk = g_io_channel_unix_get_fd(client->channel);
 
 	if (read_len == 0) {
-		DBG("client %d closed, pending %d bytes",
+		debug("client %d closed, pending %d bytes",
 			client_sk, client->buf_end);
 		g_hash_table_remove(partial_tcp_req_table,
 					GINT_TO_POINTER(client_sk));
 		return false;
 	}
 
-	DBG("client %d received %d bytes", client_sk, read_len);
+	debug("client %d received %d bytes", client_sk, read_len);
 
 	client->buf_end += read_len;
 
@@ -2984,24 +3047,24 @@ static bool read_tcp_data(struct tcp_partial_client_data *client,
 
 	msg_len = get_msg_len(client->buf);
 	if (msg_len > TCP_MAX_BUF_LEN) {
-		DBG("client %d sent too much data %d", client_sk, msg_len);
+		debug("client %d sent too much data %d", client_sk, msg_len);
 		g_hash_table_remove(partial_tcp_req_table,
 					GINT_TO_POINTER(client_sk));
 		return false;
 	}
 
 read_another:
-	DBG("client %d msg len %d end %d past end %d", client_sk, msg_len,
+	debug("client %d msg len %d end %d past end %d", client_sk, msg_len,
 		client->buf_end, client->buf_end - (msg_len + 2));
 
 	if (client->buf_end < (msg_len + 2)) {
-		DBG("client %d still missing %d bytes",
+		debug("client %d still missing %d bytes",
 			client_sk,
 			msg_len + 2 - client->buf_end);
 		return true;
 	}
 
-	DBG("client %d all data %d received", client_sk, msg_len);
+	debug("client %d all data %d received", client_sk, msg_len);
 
 	err = parse_request(client->buf + 2, msg_len,
 			query, sizeof(query));
@@ -3042,7 +3105,7 @@ read_another:
 		int ttl_left = 0;
 		struct cache_data *data;
 
-		DBG("cache hit %s type %s", query, qtype == 1 ? "A" : "AAAA");
+		debug("cache hit %s type %s", query, qtype == 1 ? "A" : "AAAA");
 		if (qtype == 1)
 			data = entry->ipv4;
 		else
@@ -3059,7 +3122,7 @@ read_another:
 			g_free(req);
 			goto out;
 		} else
-			DBG("data missing, ignoring cache for this query");
+			debug("data missing, ignoring cache for this query");
 	}
 
 	for (list = server_list; list; list = list->next) {
@@ -3114,7 +3177,7 @@ read_another:
 
 out:
 	if (client->buf_end > (msg_len + 2)) {
-		DBG("client %d buf %p -> %p end %d len %d new %d",
+		debug("client %d buf %p -> %p end %d len %d new %d",
 			client_sk,
 			client->buf + msg_len + 2,
 			client->buf, client->buf_end,
@@ -3130,12 +3193,12 @@ out:
 		 */
 		msg_len = get_msg_len(client->buf);
 		if ((msg_len + 2) == client->buf_end) {
-			DBG("client %d reading another %d bytes", client_sk,
+			debug("client %d reading another %d bytes", client_sk,
 								msg_len + 2);
 			goto read_another;
 		}
 	} else {
-		DBG("client %d clearing reading buffer", client_sk);
+		debug("client %d clearing reading buffer", client_sk);
 
 		client->buf_end = 0;
 		memset(client->buf, 0, TCP_MAX_BUF_LEN);
@@ -3197,7 +3260,7 @@ static gboolean tcp_client_event(GIOChannel *channel, GIOCondition condition,
 		if (errno == EAGAIN || errno == EWOULDBLOCK)
 			return TRUE;
 
-		DBG("client %d cannot read errno %d/%s", client_sk, -errno,
+		debug("client %d cannot read errno %d/%s", client_sk, -errno,
 			strerror(errno));
 		g_hash_table_remove(partial_tcp_req_table,
 					GINT_TO_POINTER(client_sk));
@@ -3214,7 +3277,7 @@ static gboolean client_timeout(gpointer user_data)
 
 	sock = g_io_channel_unix_get_fd(client->channel);
 
-	DBG("client %d timeout pending %d bytes", sock, client->buf_end);
+	debug("client %d timeout pending %d bytes", sock, client->buf_end);
 
 	g_hash_table_remove(partial_tcp_req_table, GINT_TO_POINTER(sock));
 
@@ -3237,7 +3300,7 @@ static bool tcp_listener_event(GIOChannel *channel, GIOCondition condition,
 	struct timeval tv;
 	fd_set readfds;
 
-	DBG("condition 0x%02x channel %p ifdata %p family %d",
+	debug("condition 0x%02x channel %p ifdata %p family %d",
 		condition, channel, ifdata, family);
 
 	if (condition & (G_IO_NVAL | G_IO_ERR | G_IO_HUP)) {
@@ -3267,9 +3330,9 @@ static bool tcp_listener_event(GIOChannel *channel, GIOCondition condition,
 	select(sk + 1, &readfds, NULL, NULL, &tv);
 	if (FD_ISSET(sk, &readfds)) {
 		client_sk = accept(sk, client_addr, client_addr_len);
-		DBG("client %d accepted", client_sk);
+		debug("client %d accepted", client_sk);
 	} else {
-		DBG("No data to read from master %d, waiting.", sk);
+		debug("No data to read from master %d, waiting.", sk);
 		return true;
 	}
 
@@ -3303,9 +3366,9 @@ static bool tcp_listener_event(GIOChannel *channel, GIOCondition condition,
 
 		client->ifdata = ifdata;
 
-		DBG("client %d created %p", client_sk, client);
+		debug("client %d created %p", client_sk, client);
 	} else {
-		DBG("client %d already exists %p", client_sk, client);
+		debug("client %d already exists %p", client_sk, client);
 	}
 
 	if (!client->buf) {
@@ -3329,11 +3392,11 @@ static bool tcp_listener_event(GIOChannel *channel, GIOCondition condition,
 	len = recv(client_sk, client->buf, TCP_MAX_BUF_LEN, 0);
 	if (len < 0) {
 		if (errno == EAGAIN || errno == EWOULDBLOCK) {
-			DBG("client %d no data to read, waiting", client_sk);
+			debug("client %d no data to read, waiting", client_sk);
 			return true;
 		}
 
-		DBG("client %d cannot read errno %d/%s", client_sk, -errno,
+		debug("client %d cannot read errno %d/%s", client_sk, -errno,
 			strerror(errno));
 		g_hash_table_remove(partial_tcp_req_table,
 					GINT_TO_POINTER(client_sk));
@@ -3341,14 +3404,14 @@ static bool tcp_listener_event(GIOChannel *channel, GIOCondition condition,
 	}
 
 	if (len < 2) {
-		DBG("client %d not enough data to read, waiting", client_sk);
+		debug("client %d not enough data to read, waiting", client_sk);
 		client->buf_end += len;
 		return true;
 	}
 
 	msg_len = get_msg_len(client->buf);
 	if (msg_len > TCP_MAX_BUF_LEN) {
-		DBG("client %d invalid message length %u ignoring packet",
+		debug("client %d invalid message length %u ignoring packet",
 			client_sk, msg_len);
 		g_hash_table_remove(partial_tcp_req_table,
 					GINT_TO_POINTER(client_sk));
@@ -3360,7 +3423,7 @@ static bool tcp_listener_event(GIOChannel *channel, GIOCondition condition,
 	 * that is the reason to -2 below.
 	 */
 	if (msg_len != (unsigned int)(len - 2)) {
-		DBG("client %d sent %d bytes but expecting %u pending %d",
+		debug("client %d sent %d bytes but expecting %u pending %d",
 			client_sk, len, msg_len + 2, msg_len + 2 - len);
 
 		client->buf_end += len;
@@ -3424,7 +3487,7 @@ static bool udp_listener_event(GIOChannel *channel, GIOCondition condition,
 	if (len < 2)
 		return true;
 
-	DBG("Received %d bytes (id 0x%04x)", len, buf[0] | buf[1] << 8);
+	debug("Received %d bytes (id 0x%04x)", len, buf[0] | buf[1] << 8);
 
 	err = parse_request(buf, len, query, sizeof(query));
 	if (err < 0 || (g_slist_length(server_list) == 0)) {
@@ -3501,7 +3564,7 @@ static GIOChannel *get_listener(int family, int protocol, int index)
 	int sk, type;
 	char *interface;
 
-	DBG("family %d protocol %d index %d", family, protocol, index);
+	debug("family %d protocol %d index %d", family, protocol, index);
 
 	switch (protocol) {
 	case IPPROTO_UDP:
@@ -3736,7 +3799,7 @@ static void destroy_listener(struct listener_data *ifdata)
 	for (list = request_list; list; list = list->next) {
 		struct request_data *req = list->data;
 
-		DBG("Dropping request (id 0x%04x -> 0x%04x)",
+		debug("Dropping request (id 0x%04x -> 0x%04x)",
 						req->srcid, req->dstid);
 		destroy_request_data(req);
 		list->data = NULL;
